# -*- coding: utf-8 -*-
# Copyright (c) 2015, Frappe Technologies Pvt. Ltd. and contributors
# For license information, please see license.txt

from __future__ import unicode_literals
import frappe
from frappe import _
from frappe.model.document import Document
from frappe.contacts.address_and_contact import load_address_and_contact, delete_contact_and_address

class BankAccount(Document):
	def onload(self):
		"""Load address and contacts in `__onload`"""
		load_address_and_contact(self)

	def on_trash(self):
		delete_contact_and_address('BankAccount', self.name)

	def validate(self):
		self.validate_company()

	def validate_company(self):
		if self.is_company_account and not self.company:
			frappe.throw(_("Company is manadatory for company account"))

@frappe.whitelist()
def make_bank_account(doctype, docname):
	doc = frappe.new_doc("Bank Account")
	doc.party_type = doctype
	doc.party = docname
	doc.is_default = 1

	return doc

@frappe.whitelist()
def get_party_bank_account(party_type, party):
	return frappe.db.get_value(party_type,
		party, 'default_bank_account')

@frappe.whitelist()
def get_bank_account_details(bank_account):
	return frappe.db.get_value("Bank Account",
		bank_account, ['account', 'bank', 'bank_account_no'], as_dict=1)
